@@ -12,7 +12,7 @@
  * of the table being marked by a NULL msgid. The po2table.sh script will do
  * this.
  *
- * Copyright 2004 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2005 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -30,7 +30,7 @@ struct msgtable_s {
 #if ENABLE_NLS
 struct msgtable_s *minigettext__gettable(char *);
 #else				/* ENABLE_NLS */
-struct msgtable_s *minigettext__gettable(char *)
+struct msgtable_s *minigettext__gettable(char *a)
 {
 	return NULL;
 }
